@@ -1,6 +1,6 @@
 /* pvm.h - Poke Virtual Machine.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023, 2024 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -229,12 +229,12 @@ pvm_val pvm_make_string_nodup (char *value);
 
 /* Make an offset PVM value.
 
-   MAGNITUDE is a PVM integral value.
+   MAGNITUDE is a PVM integral value.  It shall be of the same type
+   than the base type specified by TYPE.
 
-   UNIT is an ulong<64> PVM value specifying the unit of the offset,
-   in terms of the basic unit which is the bit.  */
+   TYPE is an offset PVM type.  */
 
-pvm_val pvm_make_offset (pvm_val magnitude, pvm_val unit);
+pvm_val pvm_make_offset (pvm_val magnitude, pvm_val type);
 
 /* Make an array PVM value.
 
@@ -263,9 +263,11 @@ pvm_val pvm_make_array (pvm_val nelem, pvm_val type);
 pvm_val pvm_make_struct (pvm_val nfields, pvm_val nmethods, pvm_val type);
 
 /* Make a closure PVM value.
-   PROGRAM is a PVM program that conforms the body of the closure.  */
+   PROGRAM is a PVM program that conforms the body of the closure.
+   NAME is either PVM_NULL or a PVM string value containing the name of
+   the program value.  */
 
-pvm_val pvm_make_cls (pvm_program program);
+pvm_val pvm_make_cls (pvm_program program, pvm_val name);
 
 /* Compare two PVM values.
 
@@ -303,7 +305,7 @@ pvm_val pvm_make_array_type (pvm_val type, pvm_val bound);
 pvm_val pvm_make_struct_type (pvm_val nfields, pvm_val name,
                               pvm_val *fnames, pvm_val *ftypes);
 
-pvm_val pvm_make_offset_type (pvm_val base_type, pvm_val unit);
+pvm_val pvm_make_offset_type (pvm_val base_type, pvm_val unit, pvm_val ref_type);
 pvm_val pvm_make_closure_type (pvm_val rtype, pvm_val nargs,
                                pvm_val *atypes);
 
@@ -596,6 +598,10 @@ pvm pvm_init (void);
 
 void pvm_shutdown (pvm pvm);
 
+/* Get the IO space context.  */
+
+ios_context pvm_ios_context (pvm apvm);
+
 /* Get the current run-time environment of PVM.  */
 
 pvm_env pvm_get_env (pvm pvm);
@@ -791,4 +797,10 @@ void pvm_print_val_with_params (pvm vm, pvm_val val,
                                 uint32_t flags,
                                 pvm_val *exit_exception);
 
+/* Register/unregister a thread whose stack that may contain PVM
+   values.  This is used for memory management.  */
+
+void pvm_register_thread (void);
+void pvm_unregister_thread (void);
+
 #endif /* ! PVM_H */
