@@ -22,8 +22,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
 
-#include <err.h>
 #include <getopt.h>
 #include <pthread.h>
 #include <unistd.h>
@@ -33,6 +33,34 @@
 #include "libpoke.h"
 #include "configmake.h"
 
+//--- err and errx implementation
+
+static void
+errx (int eval, const char *fmt, ...)
+{
+  va_list ap;
+
+  va_start (ap, fmt);
+  vfprintf (stderr, fmt, ap);
+  va_end (ap);
+  fprintf (stderr, "\n");
+  exit (eval);
+}
+
+static void
+err(int eval, const char *fmt, ...)
+{
+  va_list ap;
+
+  va_start (ap, fmt);
+  vfprintf (stderr, fmt, ap);
+  va_end (ap);
+  fprintf (stderr, ": %s\n", strerror (errno));
+  exit (eval);
+}
+
+//---
+
 pk_compiler pkc;
 struct usock *srv;
 
@@ -110,7 +138,8 @@ srvthread (void *data)
 {
   struct usock *srv = (struct usock *)data;
 
-  usock_serve (srv);
+  if (usock_serve (srv) != USOCK_SERVE_OK)
+    errx (1, "usock_serve() failed: %s\n", usock_serve_error (srv));
   return NULL;
 }
 
@@ -359,8 +388,10 @@ poked_disas_send (void)
       iteration_begin (srv, USOCK_CHAN_OUT_PDISAS, iteration);
 
       /* Send PDISAS_KIND message with the function_name/expression.  */
-      usock_out_printf (srv, USOCK_CHAN_OUT_PDISAS, PDISAS_KIND, "%c%s",
-                        (int)kind, str);
+      if (usock_out_printf (srv, USOCK_CHAN_OUT_PDISAS, PDISAS_KIND, "%c%s",
+                            (int)kind, str)
+          == -1)
+        errx (1, "poked_disas_send: usock_out_printf() failed");
 
       termout_disas ();
       if (kind == PLET_DISAS_KIND_FUNC || kind == PLET_DISAS_KIND_FUNC_NATIVE)
@@ -368,8 +399,13 @@ poked_disas_send (void)
           int native_p = kind == PLET_DISAS_KIND_FUNC_NATIVE;
 
           if (pk_disassemble_function (pkc, str, native_p) != PK_OK)
-            usock_out_printf (srv, USOCK_CHAN_OUT_PDISAS, PDISAS_ERR,
-                              "invalid function `%s' to disassemble", str);
+            {
+              if (usock_out_printf (srv, USOCK_CHAN_OUT_PDISAS, PDISAS_ERR,
+                                    "invalid function `%s' to disassemble",
+                                    str)
+                  == -1)
+                errx (1, "poked_disas_send: usock_out_printf() failed");
+            }
         }
       else if (kind == PLET_DISAS_KIND_EXPR
                || kind == PLET_DISAS_KIND_EXPR_NATIVE)
@@ -377,8 +413,13 @@ poked_disas_send (void)
           int native_p = kind == PLET_DISAS_KIND_EXPR_NATIVE;
 
           if (pk_disassemble_expression (pkc, str, native_p) != PK_OK)
-            usock_out_printf (srv, USOCK_CHAN_OUT_PDISAS, PDISAS_ERR,
-                              "invalid expression `%s' to disassemble", str);
+            {
+              if (usock_out_printf (srv, USOCK_CHAN_OUT_PDISAS, PDISAS_ERR,
+                                    "invalid expression `%s' to disassemble",
+                                    str)
+                  == -1)
+                errx (1, "poked_disas_send: usock_out_printf() failed");
+            }
         }
       termout_restore ();
 
